phase = 0
deadUnit = nil
fodderSpawned = false
numDestroyed = 0
enableVBar = false

-- Called whenever the level starts
function onLevelStart()
	ShowMessageBox("Tutorial", "You have been commanded to take a small force and scout the flank for enemy troops. \n\n So far everything is quiet. \n\n Almost too quiet.")
	
  -- Set minimal UI for tutorial
  SetMinimalUI()
  
  -- Disables default battle victory/loss conditions
	SetWinnable(false)
	
  -- Disable cinematic intro cam
	SetCinematicTimeout(-1)
	
  -- Set battle time-out to 777 seconds
  SetBattleTime( 777 )
	
  -- Hide and disable normal countdown timer
  -- SetDisplayTimer(0,0)
	
  -- Set camera x,y,z + yaw + zoom
	SetCamera(0,0,-99,30,50)
  
  -- Draw a guide arrow for first trigger
  ShowGuideArrowForTrigger("Trigger1")
end

function RoK_MessageBoxFinished()
	if phase == 0 then
		phase = 1
		onUnitDeselected()
	end
end

function EvaluateWin()
  -- only allow win if we've killed everybody and cast both types of dropped spell
  if numDestroyed == 4 and GetSpellCast("RaiseDead", 25) and GetSpellCast("Fireball", 25) then
    Win()
  end
end

-- Called whenever a dropped spell is collected
function onSpellCollected(spellName, level)

  if spellName == "Fireball" and level == 25 then
    -- Spawn a Mossmane unit (Lv25 if we've pressed the multi-select button, otherwise Lv19)
    -- HasEventHappened is now obsoltete:
    -- if HasEventHappened("MULTIPLE_SELECTION") then
    --   SpawnUnits("LevelTest", 25)
    -- else
    --   SpawnUnits("LevelTest", 19)
    -- end
    fodderSpawned = true
  end
  
  -- If we've already cast the first RaiseDead spell drop another one
  if GetSpellCast("RaiseDead", 25) then
    
    -- Will cause a guide arrow to be shown for the following call to 'DropSpell'
    ShowGuideArrowForNextSpellDrop()
    
    -- Drop Lv25 RaiseDead spell at trigger point
    DropSpell("RaiseDead", 25, "SPAWN3")
  end
  
end

-- Called whenever a spell is cast
function onSpellCast(spellName, level)
  
  -- Always kill the Mossmane when the fireball spell is cast (regardless of aim!)
  if spellName == "Fireball" and level == 25 then
    local units = GetUnits("LevelTest")
    for i,unit in pairs(units) do
			KillUnit(unit)
	  end
    
    -- Get rid of all guide arrows
    ClearAllGuideArrows()
  end
  
  if spellName == "RaiseDead" then
    -- Since we now might have multiple units, enable the multi-select button
    SetMultipleUnitSelectionButtonEnabled(true)
  end
  
  EvaluateWin()
end

-- Called whenever a unit is selected
function onUnitSelected(unit)
	if phase == 1 then
		SetTutorial("Drag from your unit to the target to give a move order. ")
		
		ForceGiveMoveOrder("Trigger1")
	else
		if phase == 2 then
			SetTutorial("Drag from a unit to the enemy to attack them.")
		end
	
		ForceGiveAttack()
	end
  
  -- This will cause a guide arrow to toggle on/off whenever a unit is selected
  if GetUnitGuideArrowVisible(unit) then
    HideUnitGuideArrow(unit)
  else
    ShowUnitGuideArrow(unit)
  end

--[[ Similar to named events (see onFirstTimeEvent), UI elements are currently referred to by their ActionScript names.
     Again, we may want to solidify this be adding more specific function to "scripting.lua" (e.g. ShowMultiSelectGuideArrow, etc...)
     
     Here are some potentially useful UI component names:
     
     "StandGuardBtn" 
     "BtnMultiselect"
     "TroopSpellList"
     "ReinforceBtn"
     "VictoryBar"
     "Timer"          
--]]
  
  -- This will toggle a guide arrow pointing to the "multi-select" button (see comment above)
  if GetUIGuideArrowVisible("BtnMultiselect") then
    HideUIGuideArrow("BtnMultiselect")
  else
    ShowUIGuideArrow("BtnMultiselect")
  end
  
  -- This will toggle the visibility of the victory bar. In this particular scenario it can
  -- only be seen when the Mossmane is spawned (tutorial units don't register any victory quota)
  SetVictoryBarEnabled(enableVBar)
  enableVBar = not enableVBar
  
  -- Whenever we select a unit, we'll drop a spoil. Spoils are always dropped on a given unit, although this is purely visual (...makes no material difference to player)
  DropSpoilOnUnit(unit)

  if deadUnit then
    -- We store the last unit which was destroyed.
    -- As a sanity check we'll check if it's really dead and display its health...
    if deadUnit.alive then
      SetTutorial("Dead unit is alive????!!! Selected health = " .. unit.health)
    else
      SetTutorial("Dead unit is actually dead  Selected health = " .. unit.health)
    end
  else
    
    -- Of all units, show how many are hidden vs how many are alive...
    local numHidden = 0
    local numAlive = 0
    local units = GetUnits("*", "*");
    for i,unit in pairs(units) do
      if unit.hidden then
        numHidden = numHidden+1
      end
      if unit.alive then
        numAlive = numAlive+1
      end
    end
    
    SetTutorial(numHidden .. " hidden units out of " .. numAlive .. " alive units")
  end
end

-- Called when a unit is deselected
function onUnitDeselected()
	if phase < 2 then
		SetTutorial("Tap on the highlighted unit to select it.")
	end
	
  -- Not totally sure what this does!
  local allies = GetUnits("*", "*", 0) -- get all attackers
	ForceSelectAlly(allies[1])
end

function RoK_OnTriggerEnter()
	if phase == 1 then
  
    -- Drop some spells in two separate trigger areas when we first enter a trigger area
    DropSpell("Fireball", 25, "SPAWN1")
    DropSpell("RaiseDead", 25, "SPAWN3")
    
    -- Enable spells UI
    SetTroopAndSpellSelectorEnabled(true)
  
		--[[SetTutorial("Enemy forces have appeared on the right flank!\nSend your troops to attack them!")]]
	    AddTimer(0.0,SetTutorial,"Enemy forces have appeared on the flank!","this popup at 0 second")
		AddTimer(4.0,SetTutorial,"Drag from a unit to the enemy to attack them.","this popup at 3 second")
		SpawnUnits("KillSingle")
		
		phase = 2
	end
end

function onUnitDestroyed(unit)
  numDestroyed = numDestroyed + 1
  deadUnit = unit
	SetTutorial("")
	
  -- Check if our Mossmane fodder is still alive...
  local fodderAlive = false
  local units = GetUnits("LevelTest")
  for i,unit in pairs(units) do
    fodderAlive = unit.alive
	end
  
  if fodderSpawned and not fodderAlive then
    -- Only spawn the remaining two dumb militia units after we've killed the Mossmane
		AddTimer(0.0,SetTutorial,"More units approach from the flank!","this popup at 0 second")
	  AddTimer(4.0,SetTutorial,"Command your troops to attack them!","this popup at 3 second")
    SpawnUnits("KillMultiple")
						
		phase = 3
	end
  
  EvaluateWin()
end