
-- used to keep track of the number if player units
playerUnits = {}

-- used to track whether the reinforcements have been called or activated yet
reinforcementsButtonShown = false
reinforcementsButtonActivated  = false
reinforcementUnitSelected = false
blockMessageShown = false

-- called at before the deploy phase
function onDeployStart()
--	ShowMessageBox("Outnumbered!", "We must survive until reinforcements arrive! \n\n try to pick off units 1 by 1!")
	
	-- register the triggers in the level
	SetTriggerCallbacks("trigger_block", onTriggerEnter_trigger_block, onTriggerLeave_trigger_block)
end

-- called after the deploy phase
function onLevelStart()
	
	-- turn off the game timer
	SetDisplayTimer(0,0)
	
	-- get the initial list of units for us to track
	playerUnits = GetUnits("*", "*", 0)
	
	-- kick off the enemySizeChecker function
	AddTimer(40,reinforcementCountdown)
	
	-- turn off the reinforcements button
	SetScriptedReinforcementsEnabled (false)
	
	
end

-- put up a message explaining what's going on and clear it after x seconds
function onCameraAnimationFinish(cameraName)
	if cameraName == "BATTLE_START" then
		ShowDialog("Portraits/Guard_talkinghead","Reinforcements are on the way, we just need to survive!")
		AddTimer(5,clearTutorialMessage)
	end
end
 
function clearTutorialMessage()
	-- clear any tutorial messages
	SetTutorial("")
end
	
function checkPlayerUnits()
	-- print out all of the units in the playerUnits array
	for i,unit in pairs(playerUnits) do
		print(unit)
	end
end

function reinforcementCountdown()
	-- when we've hit the timer then activate the reinforcement button (we also trigger the button if the player becomes too weak)
	activateReinforcementButton()
end

-- if the polayer is escaping down the back channel then recommend they use a unit to block their pursuers
function onTriggerEnter_trigger_block(unit)
	if unit.side ==0 then
		SetTutorial("Use a unit to block your pursuers and let the others escape")
		AddTimer(5,clearTutorialMessage)
		DisableTrigger("trigger_block")
	end
end

function onUnitDestroyed(deadUnit)
	-- if a friendly unit has died then remove it from the player unit array
	-- if the player's forces drop below a set amount then trigger the reinforcement button
	if deadUnit.side == 0 then
		-- for each unit in the playerUnits array
		for i,unit in pairs(playerUnits) do
			-- if this is the dead unit then remove it from the array
			if unit == deadUnit then
				-- remove the element from the array at point i
				table.remove(playerUnits,i)
				
				-- if the player's army has dropped below 2 then kick off the reinforcement button
				if #playerUnits <= 2  then
					activateReinforcementButton()
				end
				
				-- leave the for loop (because we've found the right unit)
				break
			end
		end
	end
end

function activateReinforcementButton()
	
	-- if we've not already shown the reinforcement button
	if not reinforcementsButtonShown then
		
		-- clear any old tutorial messages that might be lying around
		SetTutorial("")
		
		-- stop us from showing the reinforcements button again
		reinforcementsButtonShown = true
		
		-- show this dialogue and activate the ReinforcementDialogFinished function when it's been cleared
		ShowDialog("Portraits/Guard_talkinghead","Thank the Gods! The reinforcements have arrived!\nCall them in to join the battle!")
		ReinforcementDialogFinished()
	end
end

function ReinforcementDialogFinished()
	
	-- enable the reinforcements button on the UI
	SetScriptedReinforcementsEnabled(true)
	
	-- point an arrow at the button
	ShowUIGuideArrow("ReinforceBtn")
	
	-- put up a tutorial message telling the player to push the button
	SetTutorial("Press the reinforcement button to get more troops")
	
end

--if the player presses the reinforcements button
function onScriptedReinforce()
	
	-- if the player hasn't pressed the button yet
	if not reinforcementsButtonActivated then
	
		-- make sure we never call this stuff
		reinforcementsButtonActivated = true
		
		-- spawn the reinforcements
		SpawnUnits("reinforcements")
		
		-- clear the tutorial message
		SetTutorial("")
		
		-- turn off the reinforcements button and the arrow pointing at it
		SetScriptedReinforcementsEnabled(false)
		HideUIGuideArrow("ReinforceBtn")
		
		-- show some dialogue and call the ReinforcementsDeployedDialogFinished function when it's cleared
		ShowDialog("Portraits/Guard_talkinghead","The reinforcement units are approaching from the trees behind the enemy. \n\n They are ready to attack from the rear at your command!",ReinforcementsDeployedDialogFinished)
	end
end

function ReinforcementsDeployedDialogFinished()
	
	-- point an arrow at the new units
	ShowGuideArrowForTrigger("ReinforcementMarkerTrigger")
	
end

function onUnitSelected(selectedUnit)
	
	-- if the player selects the new units
	if reinforcementsButtonActivated and not reinforcementUnitSelected then
		if selectedUnit.spawnId == "reinforcements" then
			
			-- make sure we never call this stuff again
			reinforcementUnitSelected = true
			
			-- clear the guide arrows
			ClearAllGuideArrows()
			
		end
	end
end


