scripting = {}
scripting.timers = {}
scripting.unitsInTrigger = {}

-- used for randomised lists of things
randomThingList = {}
randomisedRandomThingList = {}
randomThingSpawnCount = {}

armyBalanceSystem_level = GetPlayerCastleLevel()
armyBalanceSystem_hasBeenReset = false

_unitMetatable_ = {
	__index = function(unit, key)
		if key=="id" then return GetUnitId(unit) end
		if key=="side" then return GetUnitSide(unit) end
		if key=="defender" then return (GetUnitSide(unit)==1) end
		if key=="attacker" then return (GetUnitSide(unit)==0) end
		if key=="health" then return GetUnitHealth(unit) end
		if key=="alive" then return (GetUnitHealth(unit) > 0.0) end
		if key=="hidden" then return GetUnitHidden(unit) end
		if key=="spawnId" then return GetUnitSpawnId(unit) end
		error("Key " .. key .. " not defined for Units")
	end,
	__newindex = function(unit, key, value)
		error("Cannot assign to units")
	end,
	__tostring = function(unit)
		return "[Unit '" .. unit.id .. "' side:" .. unit.side .. " spawnId:'" .. unit.spawnId .. "']"
	end
}

-- Override built-in lua print function
function print(param, ...)
	local msg = "" .. tostring(param)
	for i=1,arg.n do
		msg = msg .. " " .. tostring(arg[i])
	end
	LuaPrint(msg)
end

function ShowMessageBox(title, msg, callback, ...)
	scripting.messageBoxCallback = callback
	scripting.messageBoxCallbackParams = arg
	LuaShowMessageBox(title, msg)
end

function _ShowMessageBoxes(offset, argTable)
	if offset <= argTable.n then
		if offset+1 <= argTable.n and type(argTable[offset])=="string" then
			ShowMessageBox(argTable[offset], argTable[offset+1], _ShowMessageBoxes, offset+2, argTable)
		elseif type(argTable[offset])=="function" then 
			local p = {}
			for i=offset+1,argTable.n do
				p[i-offset] = argTable[i]
			end
			argTable[offset]( unpack(p) )
		end
	end
end
function ShowMessageBoxes(...)
	_ShowMessageBoxes(1, arg)
end

function AddTimer(delay, callback, ...)
	local pos = 1
	local timers = scripting.timers
	local numTimers = #timers
	local time = delay + LuaGetTime()
	if type(delay)~="number" then error("AddTimer requires a number as the first parameter") end
	if type(callback)~="function" then error("AddTimer requires a function to be passed") end
	while pos <= numTimers and timers[pos].time <= time do
		pos = pos+1
	end
	local newTimer = {time=time, fn=callback, fn_params=arg}
	table.insert(timers, pos, newTimer)
	LuaSetNextTimer(timers[1].time)
	return newTimer
end

function AddVisualTimer(delay, callback, ...)
	SetDisplayTimer(delay, 0);
	AddTimer(delay, callback, unpack(arg));
end

function CancelTimer(id)
	local timers = scripting.timers
	for i=#timers,1,-1 do
		if timers[i]==id or timers[i].fn==id then
			table.remove(timers, i)
		end
	end
end

function CancelAllTimers()
	local timers = scripting.timers
	while timers[1] do
		CancelTimer(timers[1])
	end
end

function RoK_Timer(t)
	local timers = scripting.timers
	while timers[1] and timers[1].time <= t do
		local execTimer = timers[1]
		table.remove(timers, 1)
		execTimer.fn( unpack(execTimer.fn_params) )
	end
	if timers[1] then 
		LuaSetNextTimer(timers[1].time)
	end
end

function RoK_MessageBoxFinished(id)
	local fn = scripting.messageBoxCallback
	scripting.messageBoxCallback = nil
	if fn then
		fn( unpack(scripting.messageBoxCallbackParams) )
	end
end

function _GetTriggerTable(triggerName)
	local unitsInTrigger = scripting.unitsInTrigger
	local units = unitsInTrigger[triggerName]
	if not units then 
		units = {count=0} 
		unitsInTrigger[triggerName] = units
	end
	return units
end

function CountDefendersInTrigger(triggerName) return CountUnitsInTrigger(triggerName, 1) end
function CountAttackersInTrigger(triggerName) return CountUnitsInTrigger(triggerName, 0) end
function CountUnitsInTrigger(triggerName, side)
	local units = GetUnitsInTrigger(triggerName, side)
	local qty = 0
	for i,v in pairs(units) do
	  qty = qty+1
	end
	return qty
end

--This provides the id's of all the units in the zone
function CheckDefendersInTrigger(triggerName) return CheckUnitsInTrigger(triggerName, 1) end
function CheckAttackersInTrigger(triggerName) return CheckUnitsInTrigger(triggerName, 0) end
function CheckUnitsInTrigger(triggerName, side)
	return GetUnitsInTrigger(triggerName, side)
end

function SetTriggerCallbacks(triggerName, fn, fnLeave)
	local unitsInTrigger = scripting.unitsInTrigger
	local units = _GetTriggerTable(triggerName)
	units.callback = fn
	units.callbackLeave = fnLeave
	if units.count > 0 and fn then
		for i,v in pairs(units) do
			if getmetatable(i)==_unitMetatable_ then
				fn( i )
				if units.callback ~= fn then break end
			end
		end
	end
end

function DisableTrigger(triggerName)
	SetTriggerCallbacks(triggerName, nil, nil)
end

function RoK_OnTriggerEnter(triggerName, unit)
	print("trigger enter:" .. triggerName .. " by " .. unit.id .. " side:" .. unit.side)
	local units = _GetTriggerTable(triggerName)
	units[unit] = true
	units.count = units.count + 1
	
	if units.callback then
		units.callback( unit )
	end
end

function RoK_OnTriggerLeave(triggerName, unit)
	print("trigger leave:" .. triggerName .. " by " .. unit.id .. " side:" .. unit.side)
	local units = _GetTriggerTable(triggerName)
	units[unit] = nil
	units.count = units.count - 1
	
	if units.callbackLeave then
		units.callbackLeave( unit )
	end
end

function SpawnUnits(spawnId, level)
	level = level or -1
	return SpawnUnitsInternal(spawnId, false, level)
end

function RespawnUnits(spawnId, level)
	level = level or -1
	return SpawnUnitsInternal(spawnId, true, level)
end

-- Not needed as a seperate function any more.
SpawnPlayerUnits = SpawnUnits

function SwitchBarricadesToAttacker()
	SwitchBarricades(true);
end

function SwitchBarricadesToDefender()
	SwitchBarricades(false);
end

function SwitchSideToPlayer(unit)
	SwitchSide(unit, 0)
end

function onUnitDestroyed(unit)
	print("Unit Destroyed: " , unit)
end

function ForceGiveMoveOrder(target)
  ClearTargetCircles()
  AddTargetCircle(target)
end

function string.startswith(String,Start)
   return string.sub(String,1,string.len(Start))==Start
end

function string.endswith(String,End)
   return End=='' or string.sub(String,-string.len(End))==End
end

---------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------
-- randomised list of things functionality
---------------------------------------------------------------------------------------------------------

-- (clear and) create the list of random things
function CreateRandomisedListOfThings(listName,...)
	
	-- clear and update the list
	UpdateRandomisedListOfThings(true,listName,arg)
	
end

-- add to the list (don't clear it beforehand)
function AddToRandomisedListOfThings(listName,...)
	
	-- add to the end of the list
	UpdateRandomisedListOfThings(false,listName,arg)
	
end

function GetSizeOfRandomisedList(scriptName)

	listIndex = GetRandomisedListIndex(scriptName)
	
	if listIndex ~= 0 then
		return #randomThingList[listIndex] - 1
	else
		print("URECOGNISED RANDOM LIST \"" .. scriptName .. "\"")
		return 0
	end
	
end

function UpdateRandomisedListOfThings(clearList,listName,arg)
	
	local listIndex = 1
	local isNamedList = false
	
	-- if the first entry is a list name
	if type(listName) == "string" and string.find(listName,"List_") then
		
		-- get the index of the list (or get listnumber + 1 if we've not stored this list before)
		listIndex = FindOrCreateRandomisedListIndex(listName)
		
		-- tell us we've got a list
		isNamedList = true
	end
	
	-- clear the list if we want to
	if clearList then
		randomThingList[listIndex] = {}
	end
	
	-- put the listName into the first slot (if it's a named list) or add it to the list instead
	if isNamedList then
		randomThingList[listIndex][1] = listName
	else
		-- check the first entry is empty (if new list) and add the listname to the end of the list
		randomThingList[listIndex][1] = ""
		table.insert(randomThingList[listIndex], listName)
	end
	
	--iterate through the incoming arguments and put them into the list
	for i=1,arg.n do
		table.insert(randomThingList[listIndex], arg[i])
	end
	
	-- create the randomised version of the list
	RandomiseRandomisedListOfThings(listIndex)

end

function GetRandomisedListIndex(listName)
	-- for all of the lists we have
	for i, list in pairs(randomThingList) do
	
		-- if it is a list (and not nil)
		if type(list) == "table" then
		
			-- if the first entry is a string (so we don't try to check the name against something that isn't a string)
			if type(list[1]) == "string" then
				
				-- if the name matches the list
				if listName == list[1] then
					
					-- return the index of the list
					return i
				end
			end
		end
	end
	
	--else return 0
	return 0

end

function FindOrCreateRandomisedListIndex(listName)
	
	returnListIndex = GetRandomisedListIndex(listName)
	
	if returnListIndex ~= 0 then
		return returnListIndex
	end
	
	-- otherwise return the empty slot index at the end of the list
	-- if the there is no list yet then fill the first one (stored for non-named lists) and return the one after that.
	if #randomThingList == 0 then
		randomThingList[1] = {}
		randomisedRandomThingList[1] = {}
		randomThingSpawnCount[1] = 1
		return 2
	else
		return #randomThingList + 1
	end
end


-- this function transfers all elements from randomThingList to randomisedRandomThingList in a random order
function RandomiseRandomisedListOfThings(listIndex)
	
	-- if we've not been given an index then randomise the first one
	if type(listIndex) == "nil" then
		listIndex = 1
	end
	
	-- create a local copy of the random thing list
	--------------------------------------------------------------
	-- create an empty table
	local tempRandomThingList = {}
	-- transfer each element over one by one (if we do tempRandomThingList = randomThingList then it just a pointer)
	for i = 2, #randomThingList[listIndex],1 do
		tempRandomThingList[i-1] = randomThingList[listIndex][i]
	end
	--------------------------------------------------------------
	
	
	-- clear the randomisedRandomThingList
	randomisedRandomThingList[listIndex] = {}
	
	-- put in the first item
	randomisedRandomThingList[listIndex][1] = randomThingList[listIndex][1]
	
	
	-- transfer each element from tempRandomThingList to randomisedRandomThingList in a random order
	--------------------------------------------------------------
	-- for each element in tempRandomThingList (ie a loop that goes for the number of elements in tempRandomThingList)
	for i = 1, #tempRandomThingList,1 do
		-- transfer a random element from tempRandomThingList to randomisedRandomThingList and then delete if from tempRandomThingList
		-- that way we end up with one of each element in randomisedRandomThingList but in a random order
		
		-- get a random number between 1 and the last element in the table
		local randomPosition = math.random(1,#tempRandomThingList)
		
		-- copy the element at point [randomPosition] into randomisedRandomThingList
		randomisedRandomThingList[listIndex][i+1] = tempRandomThingList[randomPosition]
		
		-- remove the element at point [randomPosition] from tempRandomThingList
		table.remove(tempRandomThingList,randomPosition)
	end
	--------------------------------------------------------------
	
	--reset the counter so we start from the start again
	randomThingSpawnCount[listIndex] = 2
end

-- get a thing out of the list and increment the list
function GetRandomisedRandomThing(listName)
	
	local listIndex = 1
	
	-- if we've been given a list name then grab get the index for it, if we've not got a list of that name then return nil
	if type(listName) == "string" and string.find(listName,"List_") then
		listIndex = FindOrCreateRandomisedListIndex(listName)
		if listIndex > #randomThingList then
			print("GetRandomisedRandomThing -- unrecognised list name!")
			return nil
		end
	end
	
	-- get the next thing string to be returned
	local thingToReturn
	if randomThingSpawnCount[listIndex] <= #randomisedRandomThingList[listIndex] then
		thingToReturn = randomisedRandomThingList[listIndex][randomThingSpawnCount[listIndex]]
	else
		thingToReturn = nil
	end
	-- increment the counter
	randomThingSpawnCount[listIndex] = randomThingSpawnCount[listIndex] + 1
	
	-- if the counter has reached the end of the randomisedRandomThingList then re-randomise randomisedRandomThingList
	if randomThingSpawnCount[listIndex] > #randomisedRandomThingList[listIndex] then
		RandomiseRandomisedListOfThings(listIndex)
	end
	
	-- if there's nothing to return then print that we've got a problem
	if type(thingToReturn) == "nil" then
		if type(listName) == "nil" then
			print("GetRandomisedRandomThing -- no unnamed list created so nothing to return")
		else
			if type(listName) == "string" then
				print("GetRandomisedRandomThing -- nothing to return from " .. listName .. "!")
			else
				print("GetRandomisedRandomThing -- nothing to return from unknown listName!")
			end
		end
	end
	
	return thingToReturn
end

---------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------
-- armyBalanceSystem functionality
---------------------------------------------------------------------------------------------------------

function SetArmyBalanceSystemLevel(newLevel)
	
	armyBalanceSystem_level = newLevel
	
	if armyBalanceSystem_hasBeenReset then
		print("Army Balance System has been set a second time - you shouldn't do this!")
	else
		print("Army Balance System set to ".. newLevel)
	end
	
end

function GetArmyBalanceSystemValue(offset)
	balanceLevel = armyBalanceSystem_level + offset
	
	if balanceLevel < 1 then
		balanceLevel = 1
	elseif balanceLevel > 25 then
		balanceLevel = 25
	end

	return balanceLevel
end

function GetArmyBalanceEasy()
	return GetArmyBalanceSystemValue(-2)
end

function GetArmyBalanceNormal()
	return GetArmyBalanceSystemValue(0)
end

function GetArmyBalanceHard()
	return GetArmyBalanceSystemValue(1)
end

---------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------